@@ -3,6 +3,8 @@
 import os
 import pandas as pd
 import dandelion as ddl
+import pytest
+from unittest.mock import patch
 from subprocess import run
 
 
@@ -20,7 +22,7 @@ def test_callscript():
 def test_container():
     """Test script to run container."""
     os.system(
-        "cd /tests; python /share/dandelion_preprocess.py --meta test.csv;"
+        "cd /tests; python /share/dandelion_preprocess.py --meta test.csv --file_prefix filtered;"
     )
     dat = pd.read_csv(
         "/tests/sample_test_10x/dandelion/filtered_contig_dandelion.tsv",
@@ -36,3 +38,13 @@ def test_container():
     except:
         pass
     assert vdj is not None
+
+
+@patch("matplotlib.pyplot.show")
+def test_threshold(mock_show):
+    """Test script to run container."""
+    os.system(
+        "cd /tests; python /share/changeo_clonotypes.py --h5ddl sample_test_10x/demo-vdj.h5ddl;"
+    )
+    dat = ddl.read_h5ddl("/tests/sample_test_10x/demo-vdj_changeo.h5ddl")
+    assert not dat.data.changeo_clone_id.empty
