@@ -1,8 +1,5 @@
 #!/usr/bin/env python
 # @Author: Kelvin
-# @Date:   2021-02-11 12:22:40
-# @Last Modified by:   Kelvin
-# @Last Modified time: 2022-11-08 15:24:59
 """core module."""
 import bz2
 import copy
@@ -29,7 +26,7 @@ from pathlib import Path
 from scanpy import logging as logg
 from textwrap import dedent
 from tqdm import tqdm
-from typing import Union, Sequence, Dict, Optional, Tuple
+from typing import Union, List, Dict, Optional, Tuple
 
 from dandelion.utilities._io import *
 from dandelion.utilities._utilities import *
@@ -41,6 +38,30 @@ class Dandelion:
 
     Main class object storing input/ouput slots for all functions.
 
+    Attributes
+    ----------
+    data : pd.DataFrame
+        AIRR formatted data.
+    germline : dict
+        dictionary of germline gene:sequence records.
+    graph : Tuple[NetworkxGraph, NetworkxGraph]
+        networkx graphs for clonotype networks.
+    layout : pd.DataFrame
+        node positions for computed graph.
+    library_type : str
+        One of "tr-ab", "tr-gd", "ig".
+    metadata : pd.DataFrame
+        AIRR data collapsed per cell.
+    n_contigs : int
+        number of contigs in `.data` slot.
+    n_obs : int
+        number of cells in `.metadata` slot.
+    querier : Query
+        internal `Query` class.
+    threshold : float
+        threshold for `define_clones`.
+    write : None
+        write method.
     """
 
     def __init__(
@@ -54,6 +75,27 @@ class Dandelion:
         library_type: Optional[Literal["tr-ab", "tr-gd", "ig"]] = None,
         **kwargs,
     ):
+        """Init method for Dandelion.
+
+        Parameters
+        ----------
+        data : Optional[pd.DataFrame], optional
+            AIRR formatted data.
+        metadata : Optional[pd.DataFrame], optional
+            AIRR data collapsed per cell.
+        germline : Optional[Dict], optional
+            dictionary of germline gene:sequence records.
+        layout : Optional[pd.DataFrame], optional
+            node positions for computed graph.
+        graph : Optional[Tuple[NetworkxGraph, NetworkxGraph]], optional
+            networkx graphs for clonotype networks.
+        initialize : bool, optional
+            whether or not to initialize `.metadata` slot.
+        library_type : Optional[Literal["tr-ab", "tr-gd", "ig"]], optional
+            One of "tr-ab", "tr-gd", "ig".
+        **kwargs
+            passed to `Dandelion.update_metadata`.
+        """
         self._data = data
         self._metadata = metadata
         self.layout = layout
@@ -95,7 +137,7 @@ class Dandelion:
             self.n_contigs = self.data.shape[0]
             if metadata is None:
                 if initialize is True:
-                    update_metadata(self, **kwargs)
+                    self.update_metadata(**kwargs)
                 try:
                     self.n_obs = self.metadata.shape[0]
                 except:
@@ -193,7 +235,7 @@ class Dandelion:
         return self.data.index
 
     @data_names.setter
-    def data_names(self, names: Sequence[str]):
+    def data_names(self, names: List[str]):
         """data names setter"""
         names = self._prep_dim_index(names, "data")
         self._set_dim_index(names, "data")
@@ -214,7 +256,7 @@ class Dandelion:
         return self.metadata.index
 
     @metadata_names.setter
-    def metadata_names(self, names: Sequence[str]):
+    def metadata_names(self, names: List[str]):
         """metadata names setter"""
         names = self._prep_dim_index(names, "metadata")
         self._set_dim_index(names, "metadata")
@@ -272,18 +314,14 @@ class Dandelion:
             if isinstance(v, pd.DataFrame):
                 v.index = value
 
-    def copy(self):
+    def copy(self) -> "Dandelion":
         """
         Performs a deep copy of all slots in `Dandelion` class.
 
-        Parameters
-        ----------
-        self : Dandelion
-            `Dandelion` object.
-
         Returns
         -------
-        a deep copy of `Dandelion` class.
+        Dandelion
+            a deep copy of `Dandelion` class.
         """
         return copy.deepcopy(self)
 
@@ -299,16 +337,12 @@ class Dandelion:
     ):
         """
         Retrieve additional data columns that are useful.
+
         Parameters
         ----------
-        self : Dandelion
-            `Dandelion` object.
-        option : Literal
+        option : Literal["all", "sequence", "mutations", "cdr3 lengths", "mutations and cdr3 lengths", ], optional
             One of 'all', 'sequence', 'mutations', 'cdr3 lengths',
             'mutations and cdr3 lengths'
-        Returns
-        -------
-        Udpated metadata.
         """
         mutations_type = ["mu_count", "mu_freq"]
         mutationsdef = [
@@ -579,53 +613,51 @@ class Dandelion:
 
         if option == "all":
             if len(mutations) > 0:
-                update_metadata(
-                    self, retrieve=mutations, retrieve_mode="split and average"
+                self.update_metadata(
+                    retrieve=mutations, retrieve_mode="split and average"
                 )
-                update_metadata(
-                    self, retrieve=mutations, retrieve_mode="average"
+                self.update_metadata(
+                    retrieve=mutations, retrieve_mode="average"
                 )
             if len(vdjlengths) > 0:
-                update_metadata(
-                    self, retrieve=vdjlengths, retrieve_mode="split and average"
+                self.update_metadata(
+                    retrieve=vdjlengths, retrieve_mode="split and average"
                 )
             if len(seqinfo) > 0:
-                update_metadata(
-                    self,
+                self.update_metadata(
                     retrieve=seqinfo,
                     retrieve_mode="split and merge",
                 )
         if option == "sequence":
             if len(seqinfo) > 0:
-                update_metadata(
-                    self,
+                self.update_metadata(
                     retrieve=seqinfo,
                     retrieve_mode="split and merge",
                 )
         if option == "mutations":
             if len(mutations) > 0:
-                update_metadata(
-                    self, retrieve=mutations, retrieve_mode="split and average"
+                self.update_metadata(
+                    retrieve=mutations, retrieve_mode="split and average"
                 )
-                update_metadata(
-                    self, retrieve=mutations, retrieve_mode="average"
+                self.update_metadata(
+                    retrieve=mutations, retrieve_mode="average"
                 )
         if option == "cdr3 lengths":
             if len(vdjlengths) > 0:
-                update_metadata(
-                    self, retrieve=vdjlengths, retrieve_mode="split and average"
+                self.update_metadata(
+                    retrieve=vdjlengths, retrieve_mode="split and average"
                 )
         if option == "mutations and cdr3 lengths":
             if len(mutations) > 0:
-                update_metadata(
-                    self, retrieve=mutations, retrieve_mode="split and average"
+                self.update_metadata(
+                    retrieve=mutations, retrieve_mode="split and average"
                 )
-                update_metadata(
-                    self, retrieve=mutations, retrieve_mode="average"
+                self.update_metadata(
+                    retrieve=mutations, retrieve_mode="average"
                 )
             if len(vdjlengths) > 0:
-                update_metadata(
-                    self, retrieve=vdjlengths, retrieve_mode="split and average"
+                self.update_metadata(
+                    retrieve=vdjlengths, retrieve_mode="split and average"
                 )
 
     @deprecated(
@@ -644,18 +676,19 @@ class Dandelion:
 
         Parameters
         ----------
-        self : Dandelion
-            `Dandelion` object.
-        corrected : dict, str, Optional
+        corrected : Optional[Union[Dict, str]], optional
             dictionary of corrected germline sequences or file path to corrected germline sequences fasta file.
-        germline : str, Optional
-            path to germline database folder. Defaults to `$GERMLINE` environmental variable.
-        org : str
+        germline : Optional[str], optional
+            path to germline database folder. Defaults to `` environmental variable.
+        org : Literal["human", "mouse"], optional
             organism of reference folder. Default is 'human'.
 
-        Returns
-        -------
-        updated germline reference diciontary in `.germline` slot.
+        Raises
+        ------
+        KeyError
+            if `GERMLINE` environmental variable is not set.
+        TypeError
+            if incorrect germline provided.
         """
         start = logg.info("Updating germline reference")
         env = os.environ.copy()
@@ -760,7 +793,7 @@ class Dandelion:
 
     def store_germline_reference(
         self,
-        corrected: Optional[Union[Dict, str]] = None,
+        corrected: Optional[Union[Dict[str, str], str]] = None,
         germline: Optional[str] = None,
         org: Literal["human", "mouse"] = "human",
     ):
@@ -769,18 +802,19 @@ class Dandelion:
 
         Parameters
         ----------
-        self : Dandelion
-            `Dandelion` object.
-        corrected : dict, str, Optional
+        corrected : Optional[Union[Dict[str, str], str]], optional
             dictionary of corrected germline sequences or file path to corrected germline sequences fasta file.
-        germline : str, Optional
-            path to germline database folder. Defaults to `$GERMLINE` environmental variable.
-        org : str
+        germline : Optional[str], optional
+            path to germline database folder. Defaults to `` environmental variable.
+        org : Literal["human", "mouse"], optional
             organism of reference folder. Default is 'human'.
 
-        Returns
-        -------
-        updated germline reference diciontary in `.germline` slot.
+        Raises
+        ------
+        KeyError
+            if `GERMLINE` environmental variable is not set.
+        TypeError
+            if incorrect germline provided.
         """
         start = logg.info("Updating germline reference")
         env = os.environ.copy()
@@ -883,13 +917,215 @@ class Dandelion:
             ),
         )
 
+    def update_metadata(
+        self,
+        retrieve: Optional[Union[List[str], str]] = None,
+        clone_key: Optional[str] = None,
+        retrieve_mode: Literal[
+            "split and unique only",
+            "merge and unique only",
+            "split and merge",
+            "split and sum",
+            "split and average",
+            "split",
+            "merge",
+            "sum",
+            "average",
+        ] = "split and merge",
+        collapse_alleles: bool = True,
+        reinitialize: bool = True,
+        verbose: bool = False,
+        by_celltype: bool = False,
+    ):
+        """
+        A `Dandelion` initialisation function to update and populate the `.metadata` slot.
+
+        Parameters
+        ----------
+        retrieve : Optional[Union[List[str], str]], optional
+            column name in `.data` slot to retrieve and update the metadata.
+        clone_key : Optional[str], optional
+            column name of clone id. None defaults to 'clone_id'.
+        retrieve_mode : Literal["split and unique only", "merge and unique only", "split and merge", "split and sum", "split and average", "split", "merge", "sum", "average", ], optional
+            one of:
+                `split and unique only`
+                    returns the retrieval splitted into two columns,
+                    i.e. one for VDJ and one for VJ chains, separated by `|` for unique elements.
+                `merge and unique only`
+                    returns the retrieval merged into one column,
+                    separated by `|` for unique elements.
+                `split and merge`
+                    returns the retrieval splitted into two columns,
+                    i.e. one for VDJ and one for VJ chains, separated by `|` for every elements.
+                `split`
+                    returns the retrieval splitted into separate columns for each contig.
+                `merge`
+                    returns the retrieval merged into one columns for each contig,
+                    separated by `|` for unique elements.
+                `split and sum`
+                    returns the retrieval sumed in the VDJ and VJ columns (separately).
+                `split and average`
+                    returns the retrieval averaged in the VDJ and VJ columns (separately).
+                `sum`
+                    returns the retrieval sumed into one column for all contigs.
+                `average`
+                    returns the retrieval averaged into one column for all contigs.
+        collapse_alleles : bool, optional
+            returns the V(D)J genes with allelic calls if False.
+        reinitialize : bool, optional
+            whether or not to reinitialize the current metadata.
+            useful when updating older versions of `dandelion` to newer version.
+        verbose : bool, optional
+            whether to print progress.
+        by_celltype : bool, optional
+            whether to return the query/update by celltype.
+
+        Raises
+        ------
+        KeyError
+            if columns provided not found in Dandelion.data.
+        ValueError
+            if missing columns in Dandelion.data.
+        """
+
+        if clone_key is None:
+            clonekey = "clone_id"
+        else:
+            clonekey = clone_key
+
+        cols = [
+            "sequence_id",
+            "cell_id",
+            "locus",
+            "productive",
+            "v_call",
+            "d_call",
+            "j_call",
+            "c_call",
+            "duplicate_count",
+            "junction",
+            "junction_aa",
+        ]
+
+        if "duplicate_count" not in self.data:
+            raise ValueError(
+                "Unable to initialize metadata due to missing keys. "
+                "Please ensure either 'umi_count' or 'duplicate_count' is in the input data."
+            )
+
+        if not all([c in self.data for c in cols]):
+            raise ValueError(
+                "Unable to initialize metadata due to missing keys. "
+                "Please ensure the input data contains all the following columns: {}".format(
+                    cols
+                )
+            )
+
+        if "sample_id" in self.data:
+            cols = ["sample_id"] + cols
+
+        if "v_call_genotyped" in self.data:
+            cols = list(
+                map(lambda x: "v_call_genotyped" if x == "v_call" else x, cols)
+            )
+
+        for c in ["sequence_id", "cell_id"]:
+            cols.remove(c)
+
+        if clonekey in self.data:
+            if not all(pd.isnull(self.data[clonekey])):
+                cols = [clonekey] + cols
+
+        metadata_status = self.metadata
+        if (metadata_status is None) or reinitialize:
+            initialize_metadata(self, cols, clonekey, collapse_alleles)
+
+        tmp_metadata = self.metadata.copy()
+
+        if retrieve is not None:
+            ret_dict = {}
+            if type(retrieve) is str:
+                retrieve = [retrieve]
+            if self.querier is None:
+                querier = Query(self.data)
+                self.querier = querier
+            else:
+                if any([r not in self.querier.data for r in retrieve]):
+                    querier = Query(self.data)
+                    self.querier = querier
+                else:
+                    querier = self.querier
+
+            if type(retrieve_mode) is str:
+                retrieve_mode = [retrieve_mode]
+                if len(retrieve) > len(retrieve_mode):
+                    retrieve_mode = [x for x in retrieve_mode for i in retrieve]
+            for ret, mode in zip(retrieve, retrieve_mode):
+                ret_dict.update(
+                    {
+                        ret: {
+                            "query": ret,
+                            "retrieve_mode": mode,
+                        }
+                    }
+                )
+
+            vdj_gene_ret = ["v_call", "d_call", "j_call"]
+
+            retrieve_ = defaultdict(dict)
+            for k, v in ret_dict.items():
+                if k in self.data.columns:
+                    if by_celltype:
+                        retrieve_[k] = querier.retrieve_celltype(**v)
+                    else:
+                        retrieve_[k] = querier.retrieve(**v)
+                else:
+                    raise KeyError(
+                        "Cannot retrieve '%s' : Unknown column name." % k
+                    )
+            ret_metadata = pd.concat(retrieve_.values(), axis=1, join="inner")
+            ret_metadata.dropna(axis=1, how="all", inplace=True)
+            for col in ret_metadata:
+                if all_missing(ret_metadata[col]):
+                    ret_metadata.drop(col, axis=1, inplace=True)
+
+            if collapse_alleles:
+                for k in ret_dict.keys():
+                    if k in vdj_gene_ret:
+                        for c in ret_metadata:
+                            if k in c:
+                                ret_metadata[c] = [
+                                    "|".join(
+                                        [
+                                            "|".join(list(set(yy.split(","))))
+                                            for yy in list(
+                                                set(
+                                                    [
+                                                        re.sub(
+                                                            "[*][0-9][0-9]",
+                                                            "",
+                                                            tx,
+                                                        )
+                                                        for tx in t.split("|")
+                                                    ]
+                                                )
+                                            )
+                                        ]
+                                    )
+                                    for t in ret_metadata[c]
+                                ]
+
+            for r in ret_metadata:
+                tmp_metadata[r] = pd.Series(ret_metadata[r])
+            self.metadata = tmp_metadata.copy()
+
     def write_pkl(self, filename: str = "dandelion_data.pkl.pbz2", **kwargs):
         """
         Writes a `Dandelion` class to .pkl format.
 
         Parameters
         ----------
-        filename
+        filename : str, optional
             path to `.pkl` file.
         **kwargs
             passed to `_pickle`.
@@ -915,14 +1151,14 @@ class Dandelion:
 
     def write_airr(self, filename: str = "dandelion_airr.tsv", **kwargs):
         """
-        Writes a `Dandelion` class to .pkl format.
+        Writes a `Dandelion` class to AIRR formatted .tsv format.
 
         Parameters
         ----------
-        filename
-            path to `.pkl` file.
+        filename : str, optional
+            path to `.tsv` file.
         **kwargs
-            passed to `_pickle`.
+            passed to `pandas.DataFrame.to_csv`.
         """
         data = sanitize_data(self.data)
         data.to_csv(filename, sep="\t", index=False, **kwargs)
@@ -967,17 +1203,22 @@ class Dandelion:
 
         Parameters
         ----------
-        filename
+        filename : str, optional
             path to `.h5` file.
-        complib : str, Optional
+        complib : Literal["zlib", "lzo", "bzip2", "blosc", "blosc:blosclz", "blosc:lz4", "blosc:lz4hc", "blosc:snappy", "blosc:zlib", "blosc:zstd", ], optional
             method for compression for data frames. see
             https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.to_hdf.html
-        compression : str, Optional
+        compression : Literal["zlib", "lzo", "bzip2", "blosc", "blosc:blosclz", "blosc:lz4", "blosc:lz4hc", "blosc:snappy", "blosc:zlib", "blosc:zstd", ], optional
             same call as complib. Just a convenience option.
-        compression_opts : {0-9}, Optional
+        compression_level : Optional[int], optional
             Specifies a compression level for data. A value of 0 disables compression.
         **kwargs
             passed to `pd.DataFrame.to_hdf`.
+
+        Raises
+        ------
+        ValueError
+            if both `complib` and `compression` are specified.
         """
         if compression_level is None:
             compression_level = 9
@@ -1106,21 +1347,26 @@ class Dandelion:
         **kwargs,
     ):
         """
-        Writes a `Dandelion` class to .h5 format.
+        Writes a `Dandelion` class to .h5ddl format.
 
         Parameters
         ----------
-        filename
-            path to `.h5` file.
-        complib : str, Optional
+        filename : str, optional
+            path to `.h5ddl` file.
+        complib : Literal["zlib", "lzo", "bzip2", "blosc", "blosc:blosclz", "blosc:lz4", "blosc:lz4hc", "blosc:snappy", "blosc:zlib", "blosc:zstd", ], optional
             method for compression for data frames. see
             https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.to_hdf.html
-        compression : str, Optional
+        compression : Literal["zlib", "lzo", "bzip2", "blosc", "blosc:blosclz", "blosc:lz4", "blosc:lz4hc", "blosc:snappy", "blosc:zlib", "blosc:zstd", ], optional
             same call as complib. Just a convenience option.
-        compression_opts : {0-9}, Optional
+        compression_level : Optional[int], optional
             Specifies a compression level for data. A value of 0 disables compression.
         **kwargs
             passed to `pd.DataFrame.to_hdf`.
+
+        Raises
+        ------
+        ValueError
+            if both `complib` and `compression` are specified.
         """
         if compression_level is None:
             compression_level = 9
@@ -1798,8 +2044,8 @@ class Query:
 
 
 def initialize_metadata(
-    self, cols: Sequence, clonekey: str, collapse_alleles: bool
-) -> Dandelion:
+    vdj_data, cols: List[str], clonekey: str, collapse_alleles: bool
+):
     """Initialize Dandelion metadata."""
     init_dict = {}
     for col in cols:
@@ -1829,21 +2075,28 @@ def initialize_metadata(
                 }
             }
         )
-    update_rearrangement_status(self)
+    update_rearrangement_status(vdj_data)
 
-    if "ambiguous" in self.data:
-        dataq = self.data[self.data["ambiguous"] == "F"]
+    if "ambiguous" in vdj_data.data:
+        dataq = vdj_data.data[vdj_data.data["ambiguous"] == "F"]
     else:
-        dataq = self.data
-    if self.querier is None:
+        dataq = vdj_data.data
+    if vdj_data.querier is None:
         querier = Query(dataq)
-        self.querier = querier
+        vdj_data.querier = querier
     else:
-        querier = self.querier
+        if vdj_data.metadata is not None:
+            if any(~vdj_data.metadata_names.isin(vdj_data.data.cell_id)):
+                querier = Query(dataq)
+                vdj_data.querier = querier
+            else:
+                querier = vdj_data.querier
+        else:
+            querier = vdj_data.querier
 
     meta_ = defaultdict(dict)
     for k, v in init_dict.copy().items():
-        if all_missing(self.data[k]):
+        if all_missing(vdj_data.data[k]):
             init_dict.pop(k)
             continue
         meta_[k] = querier.retrieve(**v)
@@ -1868,95 +2121,52 @@ def initialize_metadata(
     reqcols1 = [
         "locus_VDJ",
     ]
-    if "v_call_genotyped" in self.data:
-        reqcols2 = [
-            "locus_VJ",
-            "productive_VDJ",
-            "productive_VJ",
-            "v_call_genotyped_VDJ",
-            "d_call_VDJ",
-            "j_call_VDJ",
-            "v_call_genotyped_VJ",
-            "j_call_VJ",
-            "c_call_VDJ",
-            "c_call_VJ",
-            "junction_VDJ",
-            "junction_VJ",
-            "junction_aa_VDJ",
-            "junction_aa_VJ",
-            "v_call_genotyped_B_VDJ",
-            "d_call_B_VDJ",
-            "j_call_B_VDJ",
-            "v_call_genotyped_B_VJ",
-            "j_call_B_VJ",
-            "c_call_B_VDJ",
-            "c_call_B_VJ",
-            "productive_B_VDJ",
-            "productive_B_VJ",
-            "v_call_genotyped_abT_VDJ",
-            "d_call_abT_VDJ",
-            "j_call_abT_VDJ",
-            "v_call_genotyped_abT_VJ",
-            "j_call_abT_VJ",
-            "c_call_abT_VDJ",
-            "c_call_abT_VJ",
-            "productive_abT_VDJ",
-            "productive_abT_VJ",
-            "v_call_genotyped_gdT_VDJ",
-            "d_call_gdT_VDJ",
-            "j_call_gdT_VDJ",
-            "v_call_genotyped_gdT_VJ",
-            "j_call_gdT_VJ",
-            "c_call_gdT_VDJ",
-            "c_call_gdT_VJ",
-            "productive_gdT_VDJ",
-            "productive_gdT_VJ",
-        ]
-    else:
-        reqcols2 = [
-            "locus_VJ",
-            "productive_VDJ",
-            "productive_VJ",
-            "v_call_VDJ",
-            "d_call_VDJ",
-            "j_call_VDJ",
-            "v_call_VJ",
-            "j_call_VJ",
-            "c_call_VDJ",
-            "c_call_VJ",
-            "junction_VDJ",
-            "junction_VJ",
-            "junction_aa_VDJ",
-            "junction_aa_VJ",
-            "v_call_B_VDJ",
-            "d_call_B_VDJ",
-            "j_call_B_VDJ",
-            "v_call_B_VJ",
-            "j_call_B_VJ",
-            "c_call_B_VDJ",
-            "c_call_B_VJ",
-            "productive_B_VDJ",
-            "productive_B_VJ",
-            "v_call_abT_VDJ",
-            "d_call_abT_VDJ",
-            "j_call_abT_VDJ",
-            "v_call_abT_VJ",
-            "j_call_abT_VJ",
-            "c_call_abT_VDJ",
-            "c_call_abT_VJ",
-            "productive_abT_VDJ",
-            "productive_abT_VJ",
-            "v_call_gdT_VDJ",
-            "d_call_gdT_VDJ",
-            "j_call_gdT_VDJ",
-            "v_call_gdT_VJ",
-            "j_call_gdT_VJ",
-            "c_call_gdT_VDJ",
-            "c_call_gdT_VJ",
-            "productive_gdT_VDJ",
-            "productive_gdT_VJ",
-        ]
-
+    vcall = (
+        "v_call_genotyped" if "v_call_genotyped" in vdj_data.data else "v_call"
+    )
+    reqcols2 = [
+        "locus_VJ",
+        "productive_VDJ",
+        "productive_VJ",
+        vcall + "_VDJ",
+        "d_call_VDJ",
+        "j_call_VDJ",
+        vcall + "_VJ",
+        "j_call_VJ",
+        "c_call_VDJ",
+        "c_call_VJ",
+        "junction_VDJ",
+        "junction_VJ",
+        "junction_aa_VDJ",
+        "junction_aa_VJ",
+        vcall + "_B_VDJ",
+        "d_call_B_VDJ",
+        "j_call_B_VDJ",
+        vcall + "_B_VJ",
+        "j_call_B_VJ",
+        "c_call_B_VDJ",
+        "c_call_B_VJ",
+        "productive_B_VDJ",
+        "productive_B_VJ",
+        vcall + "_abT_VDJ",
+        "d_call_abT_VDJ",
+        "j_call_abT_VDJ",
+        vcall + "_abT_VJ",
+        "j_call_abT_VJ",
+        "c_call_abT_VDJ",
+        "c_call_abT_VJ",
+        "productive_abT_VDJ",
+        "productive_abT_VJ",
+        vcall + "_gdT_VDJ",
+        "d_call_gdT_VDJ",
+        "j_call_gdT_VDJ",
+        vcall + "_gdT_VJ",
+        "j_call_gdT_VJ",
+        "c_call_gdT_VDJ",
+        "c_call_gdT_VJ",
+        "productive_gdT_VDJ",
+        "productive_gdT_VJ",
+    ]
     reqcols = reqcols1 + reqcols2
     for rc in reqcols:
         if rc not in tmp_metadata:
@@ -1965,6 +2175,40 @@ def initialize_metadata(
         if dc in tmp_metadata:
             tmp_metadata.drop(dc, axis=1, inplace=True)
 
+    vcalldict = {
+        vcall: "v_call",
+        "d_call": "d_call",
+        "j_call": "j_call",
+        "c_call": "c_call",
+    }
+    for _call in [vcall, "d_call", "j_call", "c_call"]:
+        tmp_metadata[vcalldict[_call] + "_VDJ_main"] = [
+            return_none_call(x) for x in tmp_metadata[_call + "_VDJ"]
+        ]
+        if _call != "d_call":
+            tmp_metadata[vcalldict[_call] + "_VJ_main"] = [
+                return_none_call(x) for x in tmp_metadata[_call + "_VJ"]
+            ]
+
+    for mode in ["B", "abT", "gdT"]:
+        tmp_metadata[vcalldict[vcall] + "_" + mode + "_VDJ_main"] = [
+            return_none_call(x)
+            for x in tmp_metadata[vcall + "_" + mode + "_VDJ"]
+        ]
+        tmp_metadata["d_call_" + mode + "_VDJ_main"] = [
+            return_none_call(x) for x in tmp_metadata["d_call_" + mode + "_VDJ"]
+        ]
+        tmp_metadata["j_call_" + mode + "_VDJ_main"] = [
+            return_none_call(x) for x in tmp_metadata["j_call_" + mode + "_VDJ"]
+        ]
+        tmp_metadata[vcalldict[vcall] + "_" + mode + "_VJ_main"] = [
+            return_none_call(x)
+            for x in tmp_metadata[vcall + "_" + mode + "_VJ"]
+        ]
+        tmp_metadata["j_call_" + mode + "_VJ_main"] = [
+            return_none_call(x) for x in tmp_metadata["j_call_" + mode + "_VJ"]
+        ]
+
     if "locus_VDJ" in tmp_metadata:
         suffix_vdj = "_VDJ"
         suffix_vj = "_VJ"
@@ -2107,8 +2351,7 @@ def initialize_metadata(
     vdj_gene_calls = ["v_call", "d_call", "j_call"]
     if collapse_alleles:
         for x in vdj_gene_calls:
-
-            if x in self.data:
+            if x in vdj_data.data:
                 for c in tmp_metadata:
                     if x in c:
                         tmp_metadata[c] = [
@@ -2179,16 +2422,19 @@ def initialize_metadata(
         ref_col="chain_status",
     )
     # if metadata already exist, just overwrite the default columns?
-    if self.metadata is not None:
-        for col in tmp_metadata:
-            self.metadata[col] = pd.Series(tmp_metadata[col])
+    if vdj_data.metadata is not None:
+        if any(~vdj_data.metadata_names.isin(vdj_data.data.cell_id)):
+            vdj_data.metadata = tmp_metadata.copy()  # reindex and replace.
+        else:
+            for col in tmp_metadata:
+                vdj_data.metadata[col] = pd.Series(tmp_metadata[col])
     else:
-        self.metadata = tmp_metadata.copy()
+        vdj_data.metadata = tmp_metadata.copy()
 
 
 def update_metadata(
-    self: Dandelion,
-    retrieve: Optional[Union[Sequence, str]] = None,
+    vdj_data: Dandelion,
+    retrieve: Optional[Union[List[str], str]] = None,
     clone_key: Optional[str] = None,
     retrieve_mode: Literal[
         "split and unique only",
@@ -2205,20 +2451,20 @@ def update_metadata(
     reinitialize: bool = True,
     verbose: bool = False,
     by_celltype: bool = False,
-) -> Dandelion:
+):
     """
     A `Dandelion` initialisation function to update and populate the `.metadata` slot.
 
     Parameters
     ----------
-    self : Dandelion
-        `Dandelion` object.
-    retrieve : str, sequence, Optional
-        Column name in `.data` slot to retrieve and update the metadata.
-    clone_key : str, Optional
-        Column name of clone id. None defaults to 'clone_id'.
-    retrieve_mode: str
-        One of:
+    vdj_data : Dandelion
+        input `Dandelion` object.
+    retrieve : Optional[Union[List[str], str]], optional
+        column name in `.data` slot to retrieve and update the metadata.
+    clone_key : Optional[str], optional
+        column name of clone id. None defaults to 'clone_id'.
+    retrieve_mode : Literal["split and unique only", "merge and unique only", "split and merge", "split and sum", "split and average", "split", "merge", "sum", "average", ], optional
+        one of:
             `split and unique only`
                 returns the retrieval splitted into two columns,
                 i.e. one for VDJ and one for VJ chains, separated by `|` for unique elements.
@@ -2241,15 +2487,22 @@ def update_metadata(
                 returns the retrieval sumed into one column for all contigs.
             `average`
                 returns the retrieval averaged into one column for all contigs.
-    collapse_alleles : bool
-        Returns the V(D)J genes with allelic calls if False.
-    reinitialize : bool
-        Whether or not to reinitialize the current metadata.
-        Useful when updating older versions of `dandelion` to newer version.
-    Returns
-    -------
-    Dandelion
-        `Dandelion` object with `.metadata` slot initialized.
+    collapse_alleles : bool, optional
+        returns the V(D)J genes with allelic calls if False.
+    reinitialize : bool, optional
+        whether or not to reinitialize the current metadata.
+        useful when updating older versions of `dandelion` to newer version.
+    verbose : bool, optional
+        whether to print progress.
+    by_celltype : bool, optional
+        whether to return the query/update by celltype.
+
+    Raises
+    ------
+    KeyError
+        if columns provided not found in Dandelion.data.
+    ValueError
+        if missing columns in Dandelion.data.
     """
 
     if clone_key is None:
@@ -2271,13 +2524,13 @@ def update_metadata(
         "junction_aa",
     ]
 
-    if "duplicate_count" not in self.data:
+    if "duplicate_count" not in vdj_data.data:
         raise ValueError(
             "Unable to initialize metadata due to missing keys. "
             "Please ensure either 'umi_count' or 'duplicate_count' is in the input data."
         )
 
-    if not all([c in self.data for c in cols]):
+    if not all([c in vdj_data.data for c in cols]):
         raise ValueError(
             "Unable to initialize metadata due to missing keys. "
             "Please ensure the input data contains all the following columns: {}".format(
@@ -2285,10 +2538,10 @@ def update_metadata(
             )
         )
 
-    if "sample_id" in self.data:
+    if "sample_id" in vdj_data.data:
         cols = ["sample_id"] + cols
 
-    if "v_call_genotyped" in self.data:
+    if "v_call_genotyped" in vdj_data.data:
         cols = list(
             map(lambda x: "v_call_genotyped" if x == "v_call" else x, cols)
         )
@@ -2296,29 +2549,29 @@ def update_metadata(
     for c in ["sequence_id", "cell_id"]:
         cols.remove(c)
 
-    if clonekey in self.data:
-        if not all(pd.isnull(self.data[clonekey])):
+    if clonekey in vdj_data.data:
+        if not all(pd.isnull(vdj_data.data[clonekey])):
             cols = [clonekey] + cols
 
-    metadata_status = self.metadata
+    metadata_status = vdj_data.metadata
     if (metadata_status is None) or reinitialize:
-        initialize_metadata(self, cols, clonekey, collapse_alleles)
+        initialize_metadata(vdj_data, cols, clonekey, collapse_alleles)
 
-    tmp_metadata = self.metadata.copy()
+    tmp_metadata = vdj_data.metadata.copy()
 
     if retrieve is not None:
         ret_dict = {}
         if type(retrieve) is str:
             retrieve = [retrieve]
-        if self.querier is None:
-            querier = Query(self.data)
-            self.querier = querier
+        if vdj_data.querier is None:
+            querier = Query(vdj_data.data)
+            vdj_data.querier = querier
         else:
-            if any([r not in self.querier.data for r in retrieve]):
-                querier = Query(self.data)
-                self.querier = querier
+            if any([r not in vdj_data.querier.data for r in retrieve]):
+                querier = Query(vdj_data.data)
+                vdj_data.querier = querier
             else:
-                querier = self.querier
+                querier = vdj_data.querier
 
         if type(retrieve_mode) is str:
             retrieve_mode = [retrieve_mode]
@@ -2338,7 +2591,7 @@ def update_metadata(
 
         retrieve_ = defaultdict(dict)
         for k, v in ret_dict.items():
-            if k in self.data.columns:
+            if k in vdj_data.data.columns:
                 if by_celltype:
                     retrieve_[k] = querier.retrieve_celltype(**v)
                 else:
@@ -2379,7 +2632,7 @@ def update_metadata(
 
         for r in ret_metadata:
             tmp_metadata[r] = pd.Series(ret_metadata[r])
-        self.metadata = tmp_metadata.copy()
+        vdj_data.metadata = tmp_metadata.copy()
 
 
 def _normalize_indices(
@@ -2411,3 +2664,8 @@ def _normalize_indices(
         ax0 = _normalize_index(ax0_, names1)
         axtype = "data"
     return ax0, axtype
+
+
+def return_none_call(call: str) -> str:
+    """Return None if not present."""
+    return call.split("|")[0] if not call in ["None", ""] else "None"
