@@ -1,25 +1,26 @@
 #!/usr/bin/env python
 # @Author: kt16
-# @Date:   2020-05-12 14:01:32
-# @Last Modified by:   Kelvin
-# @Last Modified time: 2022-11-08 14:27:16
 """io module."""
 import bz2
 import gzip
 import json
 import os
 import re
+import shutil
 
 import _pickle as cPickle
+import pickle
+
+pickle.HIGHEST_PROTOCOL = 4
 import networkx as nx
 import numpy as np
 import pandas as pd
 
 from anndata import AnnData
 from collections import defaultdict, OrderedDict
-from os import PathLike
+from pathlib import Path
 from scanpy import logging as logg
-from typing import Union, Sequence, Optional, List
+from typing import Union, Optional, List
 
 from dandelion.utilities._core import *
 from dandelion.utilities._utilities import *
@@ -138,12 +139,13 @@ def read_pkl(filename: str = "dandelion_data.pkl.pbz2") -> Dandelion:
 
     Parameters
     ----------
-    filename : str
+    filename : str, optional
         path to `.pkl` file. Depending on the extension, it will try to unzip accordingly.
 
     Returns
     -------
-    Dandelion object.
+    Dandelion
+        saved `Dandelion` object in pickle format.
     """
     if isBZIP(filename):
         data = bz2.BZ2File(filename, "rb")
@@ -168,13 +170,18 @@ def read_h5(filename: str = "dandelion_data.h5") -> Dandelion:
 
     Parameters
     ----------
-    filename : str
+    filename : str, optional
         path to `.h5` file
 
     Returns
     -------
     Dandelion
         `Dandelion` object.
+
+    Raises
+    ------
+    AttributeError
+        if `data` not found in `.h5` file.
     """
     try:
         data = pd.read_hdf(filename, "data")
@@ -258,17 +265,22 @@ def read_h5(filename: str = "dandelion_data.h5") -> Dandelion:
 
 def read_h5ddl(filename: str = "dandelion_data.h5ddl") -> Dandelion:
     """
-    Read in and returns a `Dandelion` class from .h5 format.
+    Read in and returns a `Dandelion` class from .h5ddl format.
 
     Parameters
     ----------
-    filename : str
-        path to `.h5` file
+    filename : str, optional
+        path to `.h5ddl` file
 
     Returns
     -------
     Dandelion
         `Dandelion` object.
+
+    Raises
+    ------
+    AttributeError
+        if `data` not found in `.h5ddl` file.
     """
     try:
         data = pd.read_hdf(filename, "data")
@@ -362,8 +374,7 @@ def read_10x_airr(file: str) -> Dandelion:
     Returns
     -------
     Dandelion
-        `Dandelion` object of pandas data frame.
-
+        `Dandelion` object from 10x AIRR file.
     """
     dat = load_data(file)
     # get all the v,d,j,c calls
@@ -455,6 +466,10 @@ def from_scirpy(adata: AnnData) -> Dandelion:
     Dandelion
         `Dandelion` object.
 
+    Raises
+    ------
+    ImportError
+        if `scirpy` not installed.
     """
     try:
         import scirpy as ir
@@ -465,35 +480,40 @@ def from_scirpy(adata: AnnData) -> Dandelion:
 
 
 def concat(
-    arrays: Sequence[Union[pd.DataFrame, Dandelion]],
+    arrays: List[Union[pd.DataFrame, Dandelion]],
     check_unique: bool = True,
     sep: str = "-",
-    suffixes: Optional[List] = None,
-    prefixes: Optional[List] = None,
+    suffixes: Optional[List[str]] = None,
+    prefixes: Optional[List[str]] = None,
 ) -> Dandelion:
     """
     Concatenate dataframe and return as `Dandelion` object.
 
+    If both suffixes and prefixes are `None` and check_unique is True, then a sequential number suffix will be appended.
+
     Parameters
     ----------
-    arrays : Sequence
+    arrays : List[Union[pd.DataFrame, Dandelion]]
         List of `Dandelion` class objects or pandas dataframe
-    check_unique : bool
+    check_unique : bool, optional
         Check the new index for duplicates. Otherwise defer the check until necessary.
         Setting to False will improve the performance of this method.
-    sep : str
+    sep : str, optional
         the separator to append suffix/prefix.
-    suffixes : Optional, List
+    suffixes : Optional[List[str]], optional
         List of suffixes to append to sequence_id.
-        If both suffixes and prefixes are None and check_unique is True, then a sequential number suffix will be appended.
-    prefixes : Optional, List
+    prefixes : Optional[List[str]], optional
         List of prefixes to append to sequence_id.
-        If both suffixes and prefixes are None and check_unique is True, then a sequential number suffix will be appended.
 
     Returns
     -------
     Dandelion
-        `Dandelion` object
+        concatenated `Dandelion` object
+
+    Raises
+    ------
+    ValueError
+        if both prefixes and suffixes are provided.
     """
     arrays = list(arrays)
 
@@ -549,7 +569,7 @@ def concat(
 
 
 def read_10x_vdj(
-    path: Union[str, PathLike],
+    path: str,
     filename_prefix: Optional[str] = None,
     return_dandelion: bool = True,
     verbose: bool = False,
@@ -565,20 +585,25 @@ def read_10x_vdj(
 
     Parameters
     ----------
-    path : str, PathLike
+    path : str
         path to folder containing `.csv` and/or `.json` files, or path to files directly.
-    filename_prefix : str, Optional
+    filename_prefix : Optional[str], optional
         prefix of file name preceding '_contig'. None defaults to 'filtered'.
-    return_dandelion : bool
+    return_dandelion : bool, optional
         whether or not to return the output as an initialised `Dandelion` object or as a pandas `DataFrame`.
-        Default is True.
-    verbose : bool
+    verbose : bool, optional
         whether or not to print which files are read/found. Default is False.
+
     Returns
     -------
     Union[Dandelion, pd.DataFrame]
         `Dandelion` or pandas `DataFrame` object.
 
+    Raises
+    ------
+    IOError
+        if contig_annotations.csv and all_contig_annotations.json file(s) not found in the input folder.
+
     """
     if filename_prefix is None:
         filename_pre = "filtered"
@@ -754,8 +779,11 @@ def parse_json(data: list) -> defaultdict:
         else:
             continue
         for k in main_dict1.keys():
-            if data[i][k] is not None:
-                out[key].update({main_dict1[k]: data[i][k]})
+            if k in data[i]:
+                if data[i][k] is not None:
+                    out[key].update({main_dict1[k]: data[i][k]})
+                else:
+                    out[key].update({main_dict1[k]: ""})
             else:
                 out[key].update({main_dict1[k]: ""})
         if data[i]["annotations"] is not None:
@@ -786,24 +814,34 @@ def parse_json(data: list) -> defaultdict:
             if rc not in out[key]:
                 out[key].update({rc: ""})
         for k in main_dict2.keys():
-            if data[i][k] is not None:
-                out[key].update({main_dict2[k]: data[i][k]})
+            if k in data[i]:
+                if data[i][k] is not None:
+                    out[key].update({main_dict2[k]: data[i][k]})
+                else:
+                    out[key].update({main_dict2[k]: np.nan})
             else:
                 out[key].update({main_dict2[k]: np.nan})
         for rk in region_keys:
-            if data[i][rk] is not None:
-                for k in data[i][rk]:
-                    if k == "start":
-                        ka = rk + "_start"
-                    elif k == "stop":
-                        ka = rk + "_end"
-                    elif k == "nt_seq":
-                        ka = rk + ""
-                    elif k == "aa_seq":
-                        ka = rk + "_aa"
-                    else:
-                        continue
-                    out[key].update({ka: data[i][rk][k]})
+            if rk in data[i]:
+                if data[i][rk] is not None:
+                    for k in data[i][rk]:
+                        if k == "start":
+                            ka = rk + "_start"
+                        elif k == "stop":
+                            ka = rk + "_end"
+                        elif k == "nt_seq":
+                            ka = rk + ""
+                        elif k == "aa_seq":
+                            ka = rk + "_aa"
+                        else:
+                            continue
+                        out[key].update({ka: data[i][rk][k]})
+                else:
+                    for k in region_keys:
+                        out[key].update({k + "_start": np.nan})
+                        out[key].update({k + "_end": np.nan})
+                        out[key].update({k + "": ""})
+                        out[key].update({k + "_aa": ""})
             else:
                 for k in region_keys:
                     out[key].update({k + "_start": np.nan})
@@ -817,8 +855,11 @@ def parse_json(data: list) -> defaultdict:
                 else:
                     out[key].update({info_dict[info]: ""})
         for k in main_dict3.keys():
-            if data[i][k] is not None:
-                out[key].update({main_dict3[k]: data[i][k]})
+            if k in data[i]:
+                if data[i][k] is not None:
+                    out[key].update({main_dict3[k]: data[i][k]})
+                else:
+                    out[key].update({main_dict3[k]: ""})
             else:
                 out[key].update({main_dict3[k]: ""})
     return out
@@ -845,7 +886,7 @@ def parse_annotation(data: pd.DataFrame) -> defaultdict:
 
 
 def change_file_location(
-    data: Sequence, filename_prefix: Optional[Union[Sequence, str]] = None
+    data: List[str], filename_prefix: Optional[Union[List[str], str]] = None
 ):
     """
     Move file from tmp folder to dandelion folder.
@@ -854,15 +895,16 @@ def change_file_location(
 
     Parameters
     ----------
-    data : Sequence
+    data : List[str]
         list of data folders containing the .tsv files. if provided as a single string, it will first be converted to a
         list; this allows for the function to be run on single/multiple samples.
-    filename_prefix : str, Optional
+    filename_prefix : Optional[Union[List[str], str]], optional
         list of prefixes of file names preceding '_contig'. None defaults to 'filtered'.
 
-    Returns
-    -------
-    Individual V(D)J data files with v_call_genotyped column containing reassigned heavy chain v calls
+    No Longer Raises
+    ----------------
+    FileNotFoundError
+        if path to input file is not found.
     """
     fileformat = "blast"
     if type(data) is not list:
@@ -877,12 +919,6 @@ def change_file_location(
         "blast": "_igblast_db-pass.tsv",
         "airr": "_igblast_gap.tsv",
     }
-    # informat_dict2 = {
-    #     'changeo': '_igblast_db-fail.tsv',
-    #     'blast': '_igblast_db-fail.tsv',
-    #     'airr': '_igblast_gap.tsv'
-    # }
-
     filePath = None
 
     for i in range(0, len(data)):
@@ -892,39 +928,34 @@ def change_file_location(
             endswith=informat_dict[fileformat],
             subdir="tmp",
         )
-        if filePath is None:
-            raise FileNotFoundError(
-                "Path to .tsv file for {} is unknown. ".format(data[i])
-                + "Please specify path to reannotated .tsv file or folder containing reannotated .tsv file."
-            )
-        tmp = check_travdv(filePath)
-        _airrfile = filePath.replace("_db-pass.tsv", ".tsv")
-        airr_output = load_data(_airrfile)
-        cols_to_merge = [
-            "junction_aa_length",
-            "fwr1_aa",
-            "fwr2_aa",
-            "fwr3_aa",
-            "fwr4_aa",
-            "cdr1_aa",
-            "cdr2_aa",
-            "cdr3_aa",
-            "sequence_alignment_aa",
-            "v_sequence_alignment_aa",
-            "d_sequence_alignment_aa",
-            "j_sequence_alignment_aa",
-        ]
-        for x in cols_to_merge:
-            tmp[x] = pd.Series(airr_output[x])
+        if filePath is not None:
+            tmp = check_travdv(filePath)
+            _airrfile = filePath.replace("_db-pass.tsv", ".tsv")
+            airr_output = load_data(_airrfile)
+            cols_to_merge = [
+                "junction_aa_length",
+                "fwr1_aa",
+                "fwr2_aa",
+                "fwr3_aa",
+                "fwr4_aa",
+                "cdr1_aa",
+                "cdr2_aa",
+                "cdr3_aa",
+                "sequence_alignment_aa",
+                "v_sequence_alignment_aa",
+                "d_sequence_alignment_aa",
+                "j_sequence_alignment_aa",
+            ]
+            for x in cols_to_merge:
+                tmp[x] = pd.Series(airr_output[x])
 
-        write_airr(tmp, filePath)
-
-        cmd = ["rsync", "-azvh", filePath, filePath.rsplit("/", 2)[0]]
-        run(cmd)
+            write_airr(tmp, filePath)
+            fp = Path(filePath)
+            shutil.copyfile(fp, fp.parent.parent / fp.name)
 
 
 def move_to_tmp(
-    data: Sequence, filename_prefix: Optional[Union[Sequence, str]] = None
+    data: List[str], filename_prefix: Optional[Union[List[str], str]] = None
 ):
     """Move file to tmp."""
     if type(data) is not list:
@@ -943,15 +974,14 @@ def move_to_tmp(
         filePath2 = check_filepath(
             data[i], filename_prefix=filename_prefix[i], endswith=".fasta"
         )
-        cmd1 = ["mv", "-f", filePath1, filePath1.rsplit("/", 1)[0] + "/tmp"]
-        cmd2 = ["mv", "-f", filePath2, filePath2.rsplit("/", 1)[0] + "/tmp"]
-        run(cmd1)
-        run(cmd2)
+        for fp in [filePath1, filePath2]:
+            fp = Path(fp)
+            shutil.move(fp, fp.parent / "tmp" / fp.name)
 
 
 def make_all(
-    data: Sequence,
-    filename_prefix: Optional[Union[Sequence, str]] = None,
+    data: List[str],
+    filename_prefix: Optional[Union[List[str], str]] = None,
     loci: Literal["ig", "tr"] = "tr",
 ):
     """Construct db-all tsv file."""
@@ -977,6 +1007,16 @@ def make_all(
                 endswith="_igblast_db-pass_genotyped.tsv",
                 subdir="tmp",
             )
+            if filePath1 is None:
+                filePath1 = check_filepath(
+                    data[i],
+                    filename_prefix=filename_prefix[i],
+                    endswith="_igblast_db-pass.tsv",
+                    subdir="tmp",
+                )
+                out_ex = "db-pass.tsv"
+            else:
+                out_ex = "db-pass_genotyped.tsv"
         filePath2 = check_filepath(
             data[i],
             filename_prefix=filename_prefix[i],
@@ -985,8 +1025,11 @@ def make_all(
         )
         if filePath1 is not None:
             df1 = pd.read_csv(filePath1, sep="\t")
+            df1 = check_complete(df1)
+            write_airr(df1, filePath1)
             if filePath2 is not None:
                 df2 = pd.read_csv(filePath2, sep="\t")
+                df2 = check_complete(df2)
                 df = pd.concat([df1, df2])
                 if loci == "tr":
                     write_airr(
@@ -995,9 +1038,9 @@ def make_all(
                 else:
                     write_airr(
                         df,
-                        filePath1.rsplit("db-pass_genotyped.tsv")[0]
-                        + "db-all.tsv",
+                        filePath1.rsplit(out_ex)[0] + "db-all.tsv",
                     )
+                write_airr(df2, filePath2)
             else:
                 if loci == "tr":
                     write_airr(
@@ -1006,15 +1049,15 @@ def make_all(
                 else:
                     write_airr(
                         df1,
-                        filePath1.rsplit("db-pass_genotyped.tsv")[0]
-                        + "db-all.tsv",
+                        filePath1.rsplit(out_ex)[0] + "db-all.tsv",
                     )
 
 
 def rename_dandelion(
-    data: Sequence,
-    filename_prefix: Optional[Union[Sequence, str]] = None,
+    data: List[str],
+    filename_prefix: Optional[Union[List[str], str]] = None,
     endswith="_igblast_db-pass_genotyped.tsv",
+    subdir: Optional[str] = None,
 ):
     """Rename final dandlion file."""
     if type(data) is not list:
@@ -1026,12 +1069,39 @@ def rename_dandelion(
 
     for i in range(0, len(data)):
         filePath = check_filepath(
-            data[i], filename_prefix=filename_prefix[i], endswith=endswith
+            data[i],
+            filename_prefix=filename_prefix[i],
+            endswith=endswith,
+            subdir=subdir,
         )  # must be whatever's after contig
-        cmd = [
-            "mv",
-            "-f",
-            filePath,
-            filePath.rsplit(endswith)[0] + "_dandelion.tsv",
-        ]
-        run(cmd)
+        file_path = Path(filePath)
+        if subdir is None:
+            fp = filePath.rsplit(endswith)[0]
+        else:
+            fp = str(
+                file_path.parent.parent / file_path.name.rsplit(endswith)[0]
+            )
+        shutil.move(file_path, Path(fp + "_dandelion.tsv"))
+
+
+def check_complete(df: pd.DataFrame) -> pd.DataFrame:
+    """check if contig contains cdr3.
+
+    Parameters
+    ----------
+    df : pd.DataFrame
+        airr dataframe.
+
+    Returns
+    -------
+    pd.DataFrame
+        completed airr dataframe
+    """
+    if "complete_vdj" not in df:
+        df["complete_vdj"] = ""
+    for i in df.index:
+        junc = df.loc[i, "junction"]
+        if not present(junc):
+            df.at[i, "productive"] = "F"
+            df.at[i, "complete_vdj"] = "F"
+    return df
