@@ -1,8 +1,5 @@
 #!/usr/bin/env python
 # @Author: kt16
-# @Date:   2020-05-12 14:01:32
-# @Last Modified by:   Kelvin
-# @Last Modified time: 2022-11-08 14:28:06
 """utilities module."""
 import os
 import re
@@ -14,7 +11,7 @@ import pandas as pd
 from airr import RearrangementSchema
 from collections import defaultdict
 from subprocess import run
-from typing import Sequence, Tuple, Dict, Union, Optional, TypeVar, List
+from typing import Tuple, Dict, Union, Optional, TypeVar, List
 
 NetworkxGraph = TypeVar("networkx.classes.graph.Graph")
 
@@ -60,20 +57,20 @@ class Tree(defaultdict):
         self.value = value
 
 
-def dict_from_table(meta: pd.DataFrame, columns: Tuple[str, str]) -> Dict:
+def dict_from_table(meta: pd.DataFrame, columns: Tuple[str, str]) -> dict:
     """
     Generate a dictionary from a dataframe.
 
     Parameters
     ----------
-    meta
+    meta : pd.DataFrame
         pandas dataframe or file path
-    columns
+    columns : Tuple[str, str]
         column names in dataframe
 
     Returns
     -------
-    Dict
+    dict
         dictionary
     """
     if (isinstance(meta, pd.DataFrame)) & (columns is not None):
@@ -89,35 +86,35 @@ def dict_from_table(meta: pd.DataFrame, columns: Tuple[str, str]) -> Dict:
     return sample_dict
 
 
-def clean_nan_dict(d: Dict) -> Dict:
+def clean_nan_dict(d: dict) -> dict:
     """
     Remove nan from dictionary.
 
     Parameters
     ----------
-    d : Dict
+    d : dict
         dictionary
 
     Returns
     -------
-    Dict
+    dict
         dictionary with no NAs.
     """
     return {k: v for k, v in d.items() if v is not np.nan}
 
 
-def flatten(l: Sequence) -> Sequence:
+def flatten(l: list) -> list:
     """
     Flatten a list-in-list-in-list.
 
     Parameters
     ----------
-    l : Sequence
+    l : list
         a list-in-list list
 
-    Returns
-    -------
-    Sequence
+    Yields
+    ------
+    list
         a flattened list.
     """
     for el in l:
@@ -148,8 +145,9 @@ def bh(pvalues: np.array) -> np.array:
 
     Parameters
     ----------
-        pvalues : np.array
-            array of p-values to correct
+    pvalues : np.array
+        array of p-values to correct
+
     Returns
     -------
     np.array
@@ -566,12 +564,19 @@ def load_data(obj: Optional[Union[pd.DataFrame, str]]) -> pd.DataFrame:
 
     Parameters
     ----------
-    obj : DataFrame, str
+    obj : Optional[Union[pd.DataFrame, str]]
         file path to .tsv file or pandas DataFrame object.
 
     Returns
     -------
-    pandas DataFrame object.
+    pd.DataFrame
+
+    Raises
+    ------
+    FileNotFoundError
+        if input is not found.
+    KeyError
+        if `sequence_id` not found in input.
     """
     if obj is not None:
         if os.path.isfile(str(obj)):
@@ -642,25 +647,20 @@ def mask_dj(data, filename_prefix, d_evalue_threshold, j_evalue_threshold):
             filename_prefix=filename_prefix[i],
             endswith="_igblast_db-pass.tsv",
         )
-        if filePath is None:
-            raise FileNotFoundError(
-                "Path to .tsv file for {} is unknown. ".format(data[i])
-                + "Please specify path to reannotated .tsv file or folder containing reannotated .tsv file."
-            )
+        if filePath is not None:
+            dat = load_data(filePath)
+            if "d_support_blastn" in dat:
+                dat["d_call"] = [
+                    "" if s > d_evalue_threshold else c
+                    for c, s in zip(dat["d_call"], dat["d_support_blastn"])
+                ]
+            if "j_support_blastn" in dat:
+                dat["j_call"] = [
+                    "" if s > j_evalue_threshold else c
+                    for c, s in zip(dat["j_call"], dat["j_support_blastn"])
+                ]
 
-        dat = load_data(filePath)
-        if "d_support_blastn" in dat:
-            dat["d_call"] = [
-                "" if s > d_evalue_threshold else c
-                for c, s in zip(dat["d_call"], dat["d_support_blastn"])
-            ]
-        if "j_support_blastn" in dat:
-            dat["j_call"] = [
-                "" if s > j_evalue_threshold else c
-                for c, s in zip(dat["j_call"], dat["j_support_blastn"])
-            ]
-
-        write_airr(dat, filePath)
+            write_airr(dat, filePath)
 
 
 def write_airr(data, save):
@@ -771,42 +771,24 @@ def format_locus(
     """Extract locus call value from data."""
     locus_1 = dict(metadata["locus" + suffix_vdj])
     locus_2 = dict(metadata["locus" + suffix_vj])
-    productive_1 = dict(metadata["productive" + suffix_vdj])
-    productive_2 = dict(metadata["productive" + suffix_vj])
     constant_1 = dict(metadata["isotype_status"])
 
     locus_dict = {}
     for i in metadata.index:
-        pro1 = {
-            e: p
-            for e, p in enumerate([pp for pp in productive_1[i].split("|")])
-        }
         loc1 = {
             e: l for e, l in enumerate([ll for ll in locus_1[i].split("|")])
         }
-        pro2 = {
-            e: p
-            for e, p in enumerate([pp for pp in productive_2[i].split("|")])
-        }
         loc2 = {
             e: l for e, l in enumerate([ll for ll in locus_2[i].split("|")])
         }
         loc1x, loc2x = [], []
-        if not all([px == "None" for px in pro1.values()]):
-            if all([px_ != "F" for px_ in pro1.values()]):
-                for j in pro1:
-                    if pro1[j] in TRUES:
-                        loc1x.append(loc1[j])
-                loc1xx = loc1x
-                loc1x = [ij[:2] for ij in loc1x]
+        if not all([px == "None" for px in loc1.values()]):
+            loc1xx = list(loc1.values())
+            loc1x = [ij[:2] for ij in loc1.values()]
 
-        if not all([px == "None" for px in pro2.values()]):
-            if all([px_ != "F" for px_ in pro1.values()]):
-                for j in pro2:
-                    if pro2[j] in TRUES:
-                        loc2x.append(loc2[j])
-                loc2xx = loc2x
-                loc2x = [ij[:2] for ij in loc2x]
+        if not all([px == "None" for px in loc2.values()]):
+            loc2xx = list(loc2.values())
+            loc2x = [ij[:2] for ij in loc2.values()]
 
         if len(loc1x) > 0:
             if len(list(set(loc1x))) > 1:
@@ -873,7 +855,8 @@ def format_locus(
                 locus_dict.update({i: "Orphan " + tmp2})
             elif tmp2 == "None":
                 locus_dict.update({i: "Orphan " + tmp1})
-
+        if any(re.search("No_contig", tmp) for tmp in [tmp1, tmp2]):
+            locus_dict.update({i: "No_contig"})
     result = pd.Series(locus_dict)
     return result
 
