@@ -105,15 +105,11 @@ def test_readwrite10xairr(create_testfolder):
     airr_file2 = str(create_testfolder) + "/test_airr_rearrangements2.tsv"
     vdj = ddl.read_10x_airr(airr_file)
     assert vdj.data.shape[0] == 9
-    assert vdj.data.shape[1] == 32
     assert vdj.metadata.shape[0] == 5
-    assert vdj.metadata.shape[1] == 27
     vdj.write_airr(airr_file2)
     vdj2 = ddl.read_10x_airr(airr_file2)
     assert vdj2.data.shape[0] == 9
-    assert vdj2.data.shape[1] == 32
     assert vdj2.metadata.shape[0] == 5
-    assert vdj2.metadata.shape[1] == 27
     os.remove(airr_file2)
 
 
@@ -125,9 +121,7 @@ def test_read10xvdj_json(create_testfolder, json_10x_cr6):
         json.dump(json_10x_cr6, outfile)
     vdj = ddl.read_10x_vdj(json_file)
     assert vdj.data.shape[0] == 26
-    assert vdj.data.shape[1] == 49
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 31
     os.remove(json_file)
 
 
@@ -146,16 +140,12 @@ def test_read10xvdj_cr6(
     annotation_10x_cr6.to_csv(annot_file, index=False)
     vdj = ddl.read_10x_vdj(annot_file)
     assert vdj.data.shape[0] == 26
-    assert vdj.data.shape[1] == 31
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 31
     with open(json_file, "w") as outfile:
         json.dump(json_10x_cr6, outfile)
     vdj = ddl.read_10x_vdj(annot_file)
     assert vdj.data.shape[0] == 26
-    assert vdj.data.shape[1] == 49
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 31
     assert not vdj.data.sequence.empty
     os.remove(json_file)
     fh = open(fasta_file, "w")
@@ -166,9 +156,7 @@ def test_read10xvdj_cr6(
         ddl.utl.Write_output(out, fasta_file)
     vdj = ddl.read_10x_vdj(annot_file)
     assert vdj.data.shape[0] == 26
-    assert vdj.data.shape[1] == 32
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 31
     assert not vdj.data.sequence.empty
     os.remove(fasta_file)
 
@@ -183,9 +171,7 @@ def test_read10xvdj(create_testfolder, annotation_10x, fasta_10x):
     annotation_10x.to_csv(annot_file, index=False)
     vdj = ddl.read_10x_vdj(annot_file)
     assert vdj.data.shape[0] == 9
-    assert vdj.data.shape[1] == 18
     assert vdj.metadata.shape[0] == 5
-    assert vdj.metadata.shape[1] == 29
     fh = open(fasta_file, "w")
     fh.close()
     out = ""
@@ -194,9 +180,7 @@ def test_read10xvdj(create_testfolder, annotation_10x, fasta_10x):
         ddl.utl.Write_output(out, fasta_file)
     vdj = ddl.read_10x_vdj(annot_file)
     assert vdj.data.shape[0] == 9
-    assert vdj.data.shape[1] == 19
     assert vdj.metadata.shape[0] == 5
-    assert vdj.metadata.shape[1] == 29
     assert not vdj.data.sequence.empty
     os.remove(fasta_file)
 
@@ -216,16 +200,12 @@ def test_read10xvdj_cr6_folder(
     annotation_10x_cr6.to_csv(annot_file, index=False)
     vdj = ddl.read_10x_vdj(str(create_testfolder))
     assert vdj.data.shape[0] == 26
-    assert vdj.data.shape[1] == 31
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 31
     with open(json_file, "w") as outfile:
         json.dump(json_10x_cr6, outfile)
     vdj = ddl.read_10x_vdj(str(create_testfolder), filename_prefix="test_all")
     assert vdj.data.shape[0] == 26
-    assert vdj.data.shape[1] == 49
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 31
     assert not vdj.data.sequence.empty
     os.remove(json_file)
     fh = open(fasta_file, "w")
@@ -236,9 +216,7 @@ def test_read10xvdj_cr6_folder(
         ddl.utl.Write_output(out, fasta_file)
     vdj = ddl.read_10x_vdj(str(create_testfolder))
     assert vdj.data.shape[0] == 26
-    assert vdj.data.shape[1] == 32
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 31
     assert not vdj.data.sequence.empty
     os.remove(fasta_file)
 
@@ -253,9 +231,7 @@ def test_read10xvdj_folder(create_testfolder, annotation_10x, fasta_10x):
     annotation_10x.to_csv(annot_file, index=False)
     vdj = ddl.read_10x_vdj(str(create_testfolder))
     assert vdj.data.shape[0] == 9
-    assert vdj.data.shape[1] == 18
     assert vdj.metadata.shape[0] == 5
-    assert vdj.metadata.shape[1] == 29
     fh = open(fasta_file, "w")
     fh.close()
     out = ""
@@ -264,9 +240,7 @@ def test_read10xvdj_folder(create_testfolder, annotation_10x, fasta_10x):
         ddl.utl.Write_output(out, fasta_file)
     vdj = ddl.read_10x_vdj(str(create_testfolder))
     assert vdj.data.shape[0] == 9
-    assert vdj.data.shape[1] == 19
     assert vdj.metadata.shape[0] == 5
-    assert vdj.metadata.shape[1] == 29
     assert not vdj.data.sequence.empty
     os.remove(fasta_file)
 
@@ -281,12 +255,9 @@ def test_to_scirpy(create_testfolder, annotation_10x, fasta_10x):
     annotation_10x.to_csv(annot_file, index=False)
     vdj = ddl.read_10x_vdj(str(create_testfolder))
     assert vdj.data.shape[0] == 9
-    assert vdj.data.shape[1] == 18
     assert vdj.metadata.shape[0] == 5
-    assert vdj.metadata.shape[1] == 29
     adata = ddl.to_scirpy(vdj)
     assert adata.obs.shape[0] == 5
-    assert adata.obs.shape[1] == 43
     fh = open(fasta_file, "w")
     fh.close()
     out = ""
@@ -295,13 +266,10 @@ def test_to_scirpy(create_testfolder, annotation_10x, fasta_10x):
         ddl.utl.Write_output(out, fasta_file)
     vdj = ddl.read_10x_vdj(str(create_testfolder))
     assert vdj.data.shape[0] == 9
-    assert vdj.data.shape[1] == 19
     assert vdj.metadata.shape[0] == 5
-    assert vdj.metadata.shape[1] == 29
     assert not vdj.data.sequence.empty
     adata = ddl.to_scirpy(vdj)
     assert adata.obs.shape[0] == 5
-    assert adata.obs.shape[1] == 43
     os.remove(fasta_file)
     vdjx = ddl.from_scirpy(adata)
     assert vdjx.data.shape[0] == 9
@@ -319,12 +287,9 @@ def test_tofro_scirpy_cr6(create_testfolder, annotation_10x_cr6, json_10x_cr6):
         json.dump(json_10x_cr6, outfile)
     vdj = ddl.read_10x_vdj(str(create_testfolder), filename_prefix="test_all")
     assert vdj.data.shape[0] == 26
-    assert vdj.data.shape[1] == 31
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 31
     adata = ddl.to_scirpy(vdj)
     assert adata.obs.shape[0] == 10
-    assert adata.obs.shape[1] == 43
     vdjx = ddl.from_scirpy(adata)
     assert vdjx.data.shape[0] == 26
 
@@ -341,11 +306,28 @@ def test_tofro_scirpy_cr6_transfer(
         json.dump(json_10x_cr6, outfile)
     vdj = ddl.read_10x_vdj(str(create_testfolder), filename_prefix="test_all")
     assert vdj.data.shape[0] == 26
-    assert vdj.data.shape[1] == 31
     assert vdj.metadata.shape[0] == 10
-    assert vdj.metadata.shape[1] == 31
     adata = ddl.to_scirpy(vdj, transfer=True)
     assert adata.obs.shape[0] == 10
-    assert adata.obs.shape[1] == 74
     vdjx = ddl.from_scirpy(adata)
     assert vdjx.data.shape[0] == 26
+
+
+@pytest.mark.usefixtures("airr_generic")
+def test_librarytype(airr_generic):
+    """test library type"""
+    tmp = ddl.Dandelion(airr_generic)
+    assert tmp.data.shape[0] == 105
+    assert tmp.metadata.shape[0] == 40
+
+    tmp = ddl.Dandelion(airr_generic, library_type="ig")
+    assert tmp.data.shape[0] == 59
+    assert tmp.metadata.shape[0] == 20
+
+    tmp = ddl.Dandelion(airr_generic, library_type="tr-ab")
+    assert tmp.data.shape[0] == 29
+    assert tmp.metadata.shape[0] == 20
+
+    tmp = ddl.Dandelion(airr_generic, library_type="tr-gd")
+    assert tmp.data.shape[0] == 17
+    assert tmp.metadata.shape[0] == 15
