@@ -32,6 +32,12 @@ def test_formatfasta(create_testfolder):
     assert len(list((create_testfolder / "dandelion").iterdir())) == 2
 
 
+@pytest.mark.usefixtures("database_paths_mouse")
+def test_updateblastdb(database_paths_mouse):
+    """test update blast"""
+    ddl.utl.makeblastdb(database_paths_mouse["blastdb_fasta"])
+
+
 @pytest.mark.usefixtures("create_testfolder", "database_paths_mouse")
 def test_reannotategenes_original(create_testfolder, database_paths_mouse):
     """test_reannotategenes_original"""
@@ -43,7 +49,7 @@ def test_reannotategenes_original(create_testfolder, database_paths_mouse):
         flavour="original",
         org="mouse",
     )
-    assert len(list((create_testfolder / "dandelion/tmp").iterdir())) == 4
+    assert len(list((create_testfolder / "dandelion" / "tmp").iterdir())) == 4
 
 
 @pytest.mark.usefixtures("create_testfolder", "database_paths_mouse")
@@ -57,4 +63,4 @@ def test_reannotategenes_other(create_testfolder, database_paths_mouse):
         extended=False,
         org="mouse",
     )
-    assert len(list((create_testfolder / "dandelion/tmp").iterdir())) == 6
+    assert len(list((create_testfolder / "dandelion" / "tmp").iterdir())) == 6
